from dataclasses import dataclass
from os import getenv
from typing import Union

from ._constants import DEFAULT_RAISE_FOR_STATUS, DEFAULT_RETRIES, DEFAULT_TIMEOUT
from ._types import UndefinedType
from ._utils import UNDEFINED, is_nullish_str, is_undefined, str_to_bool


@dataclass
class Config:
    account_name: Union[str, UndefinedType] = UNDEFINED
    app_key: Union[str, UndefinedType] = UNDEFINED
    app_token: Union[str, UndefinedType] = UNDEFINED
    timeout: Union[float, int, None, UndefinedType] = UNDEFINED
    retries: Union[int, UndefinedType] = UNDEFINED
    raise_for_status: Union[bool, UndefinedType] = UNDEFINED

    def __init__(
        self,
        account_name: Union[str, UndefinedType] = UNDEFINED,
        app_key: Union[str, UndefinedType] = UNDEFINED,
        app_token: Union[str, UndefinedType] = UNDEFINED,
        timeout: Union[float, int, None, UndefinedType] = UNDEFINED,
        retries: Union[int, UndefinedType] = UNDEFINED,
        raise_for_status: Union[bool, UndefinedType] = UNDEFINED,
    ) -> None:
        self.account_name = self._parse_account_name(account_name)
        self.app_key = self._parse_app_key(app_key)
        self.app_token = self._parse_app_token(app_token)
        self.timeout = self._parse_timeout(timeout)
        self.retries = self._parse_retries(retries)
        self.raise_for_status = self._parse_raise_for_status(raise_for_status)

    def with_overrides(
        self,
        account_name: Union[str, UndefinedType] = UNDEFINED,
        app_key: Union[str, UndefinedType] = UNDEFINED,
        app_token: Union[str, UndefinedType] = UNDEFINED,
        timeout: Union[float, int, None, UndefinedType] = UNDEFINED,
        retries: Union[int, UndefinedType] = UNDEFINED,
        raise_for_status: Union[bool, UndefinedType] = UNDEFINED,
    ) -> "Config":
        return Config(
            account_name=(
                self.account_name if is_undefined(account_name) else account_name
            ),
            app_key=self.app_key if is_undefined(app_key) else app_key,
            app_token=self.app_token if is_undefined(app_token) else app_token,
            timeout=self.timeout if is_undefined(timeout) else timeout,
            retries=self.retries if is_undefined(retries) else retries,
            raise_for_status=(
                self.raise_for_status
                if is_undefined(raise_for_status)
                else raise_for_status
            ),
        )

    def get_account_name(self) -> str:
        if is_undefined(self.account_name):
            raise ValueError("Missing VTEX Account Name")

        return self.account_name  # type: ignore[return-value]

    def get_app_key(self) -> str:
        if is_undefined(self.app_key):
            raise ValueError("Missing VTEX APP Key")

        return self.app_key  # type: ignore[return-value]

    def get_app_token(self) -> str:
        if is_undefined(self.app_token):
            raise ValueError("Missing VTEX APP Token")

        return self.app_token  # type: ignore[return-value]

    def get_timeout(self) -> Union[float, int, None]:
        if is_undefined(self.timeout):
            return DEFAULT_TIMEOUT

        return self.timeout  # type: ignore[return-value]

    def get_retries(self) -> int:
        if is_undefined(self.retries):
            return DEFAULT_RETRIES

        return self.retries  # type: ignore[return-value]

    def get_raise_for_status(self) -> bool:
        if is_undefined(self.raise_for_status):
            return DEFAULT_RAISE_FOR_STATUS

        return self.raise_for_status  # type: ignore[return-value]

    def _parse_account_name(
        self,
        account_name: Union[str, UndefinedType] = UNDEFINED,
    ) -> Union[str, UndefinedType]:
        if isinstance(account_name, str) and account_name:
            return account_name

        if is_undefined(account_name):
            env_account_name = getenv("VTEX_ACCOUNT_NAME", UNDEFINED)

            if is_undefined(env_account_name) or env_account_name:
                return env_account_name

            raise ValueError(f"Invalid value for VTEX_ACCOUNT_NAME: {env_account_name}")

        raise ValueError(f"Invalid value for account_name: {account_name}")

    def _parse_app_key(
        self,
        app_key: Union[str, UndefinedType] = UNDEFINED,
    ) -> Union[str, UndefinedType]:
        if isinstance(app_key, str) and app_key:
            return app_key

        if is_undefined(app_key):
            env_app_key = getenv("VTEX_APP_KEY", UNDEFINED)

            if is_undefined(env_app_key) or env_app_key:
                return env_app_key

            raise ValueError(f"Invalid value for VTEX_APP_KEY: {env_app_key}")

        raise ValueError(f"Invalid value for app_key: {app_key}")

    def _parse_app_token(
        self,
        app_token: Union[str, UndefinedType] = UNDEFINED,
    ) -> Union[str, UndefinedType]:
        if isinstance(app_token, str) and app_token:
            return app_token

        if is_undefined(app_token):
            env_app_token = getenv("VTEX_APP_TOKEN", UNDEFINED)

            if is_undefined(env_app_token) or env_app_token:
                return env_app_token

            raise ValueError(f"Invalid value for VTEX_APP_TOKEN: {env_app_token}")

        raise ValueError(f"Invalid value for app_token: {app_token}")

    def _parse_timeout(
        self,
        timeout: Union[float, int, None, UndefinedType] = UNDEFINED,
    ) -> Union[float, int, None, UndefinedType]:
        if isinstance(timeout, (float, int)) and timeout > 0:
            return float(timeout)

        if timeout is None:
            return timeout

        if is_undefined(timeout):
            env_timeout = getenv("VTEX_TIMEOUT", UNDEFINED)

            if is_undefined(env_timeout):
                return env_timeout  # type: ignore[return-value]

            if is_nullish_str(env_timeout):  # type: ignore[arg-type]
                return None

            if env_timeout.isnumeric():  # type: ignore[union-attr]
                return float(env_timeout)  # type: ignore[arg-type]

            raise ValueError(f"Invalid value for VTEX_TIMEOUT: {env_timeout}")

        raise ValueError(f"Invalid value for timeout: {timeout}")

    def _parse_retries(
        self,
        retries: Union[int, UndefinedType] = UNDEFINED,
    ) -> Union[int, UndefinedType]:
        if isinstance(retries, int) and retries >= 0:
            return retries

        if is_undefined(retries):
            env_retries = getenv("VTEX_RETRIES", UNDEFINED)

            if is_undefined(env_retries):
                return env_retries  # type: ignore[return-value]

            if env_retries.isnumeric():  # type: ignore[union-attr]
                return int(round(float(env_retries), 0))  # type: ignore[arg-type]

            raise ValueError(f"Invalid value for VTEX_RETRIES: {env_retries}")

        raise ValueError(f"Invalid value for retries: {retries}")

    def _parse_raise_for_status(
        self,
        raise_for_status: Union[bool, UndefinedType] = UNDEFINED,
    ) -> Union[bool, UndefinedType]:
        if isinstance(raise_for_status, bool):
            return raise_for_status

        if is_undefined(raise_for_status):
            env_raise_for_status = getenv("VTEX_RAISE_FOR_STATUS", UNDEFINED)

            if is_undefined(env_raise_for_status):
                return env_raise_for_status  # type: ignore[return-value]

            try:
                self.raise_for_status = str_to_bool(str(env_raise_for_status))
            except ValueError:
                raise ValueError(
                    f"Invalid value for VTEX_RAISE_FOR_STATUS: {env_raise_for_status}"
                ) from None

        raise ValueError(f"Invalid value for raise_for_status: {raise_for_status}")



from logging import Logger, getLogger

from ._utils import to_snake_case


def get_child_logger(logger: Logger, name: str) -> Logger:
    return logger.getChild(to_snake_case(name))


def get_logger(name: str) -> Logger:
    return getLogger(f"vtex.{to_snake_case(name)}")



# Vtex Auth
APP_KEY_HEADER = "X-VTEX-API-AppKey"
APP_TOKEN_HEADER = "X-VTEX-API-AppToken"  # noqa: S105

# Request Defaults
DEFAULT_TIMEOUT = 10
DEFAULT_RETRIES = 5
DEFAULT_RAISE_FOR_STATUS = False

# Retry
RETRIABLE_STATUSES = {
    401, 407, 408, 409, 425, 429, 500, 502, 503, 504, 507, 511, 525,
}

# Generics
MIN_PAGE_SIZE = 1

# Logistics
LIST_CARRIERS_START_PAGE = 1
LIST_CARRIERS_MAX_PAGE_SIZE = 1000

LIST_DOCKS_START_PAGE = 1
LIST_DOCKS_MAX_PAGE_SIZE = 1000

LIST_SHIPPING_POLICIES_START_PAGE = 1
LIST_SHIPPING_POLICIES_MAX_PAGE_SIZE = 1000

# Orders
LIST_FEED_ORDERS_MAX_PAGE_SIZE = 10
LIST_ORDERS_START_PAGE = 1
LIST_ORDERS_MAX_PAGE = 30
LIST_ORDERS_MAX_PAGE_SIZE = 100

# Transactions
LIST_TRANSACTION_INTERACTIONS_RETRIES = 10



from datetime import datetime
from typing import Any, Dict, Sequence, Union

from .._constants import (
    LIST_FEED_ORDERS_MAX_PAGE_SIZE,
    LIST_ORDERS_MAX_PAGE,
    LIST_ORDERS_MAX_PAGE_SIZE,
    LIST_ORDERS_START_PAGE,
    MIN_PAGE_SIZE,
)
from .._dto import VTEXListResponse, VTEXPaginatedListResponse, VTEXResponse
from .._types import UndefinedType
from .._utils import UNDEFINED, is_undefined, now, three_years_ago
from .base import BaseAPI


class OrdersAPI(BaseAPI):
    """
    Client for the Orders API.
    https://developers.vtex.com/docs/api-reference/orders-api
    """

    ENVIRONMENT = "vtexcommercestable"

    def list_orders(
        self,
        query: Union[str, UndefinedType] = UNDEFINED,
        search: Union[str, UndefinedType] = UNDEFINED,
        creation_date_from: Union[datetime, UndefinedType] = UNDEFINED,
        creation_date_to: Union[datetime, UndefinedType] = UNDEFINED,
        incomplete: bool = False,
        order_by: str = "creationDate,desc",
        page: int = LIST_ORDERS_START_PAGE,
        page_size: int = LIST_ORDERS_MAX_PAGE_SIZE,
        **kwargs: Any,
    ) -> VTEXPaginatedListResponse:
        if page > LIST_ORDERS_MAX_PAGE:
            raise ValueError("List Orders endpoint can only return up to page 30")

        params: Dict[str, Union[str, int]] = {
            "incompleteOrders": incomplete,
            "orderBy": order_by,
            "page": max(
                min(page, LIST_ORDERS_MAX_PAGE),
                LIST_ORDERS_START_PAGE,
            ),
            "per_page": max(
                min(page_size, LIST_ORDERS_MAX_PAGE_SIZE),
                MIN_PAGE_SIZE,
            ),
        }

        if not is_undefined(query):
            params["q"] = str(query)

        if not is_undefined(search):
            params["searchField"] = str(search)

        if not is_undefined(creation_date_from) or not is_undefined(creation_date_to):
            if not isinstance(creation_date_from, datetime):
                creation_date_from = three_years_ago()

            if not isinstance(creation_date_to, datetime):
                creation_date_to = now()

            start = creation_date_from.isoformat(timespec="milliseconds").split("+")[0]
            end = creation_date_to.isoformat(timespec="milliseconds").split("+")[0]

            params["f_creationDate"] = f"creationDate:[{start}Z TO {end}Z]"

        response = self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/oms/pvt/orders/",
            params=params,
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXPaginatedListResponse,
        )

        pagination = response.pagination
        if isinstance(pagination.next_page, int) and pagination.next_page > 30:
            pagination.next_page = None

        return response

    def get_order(self, order_id: str, **kwargs: Any) -> VTEXResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint=f"/api/oms/pvt/orders/{order_id}",
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )

    def list_feed_orders(
        self,
        page_size: int = LIST_FEED_ORDERS_MAX_PAGE_SIZE,
        **kwargs: Any,
    ) -> VTEXListResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/orders/feed/",
            params={
                "maxlot": max(
                    min(page_size, LIST_FEED_ORDERS_MAX_PAGE_SIZE),
                    MIN_PAGE_SIZE,
                ),
            },
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXListResponse,
        )

    def commit_feed_orders(self, handles: Sequence[str], **kwargs: Any) -> VTEXResponse:
        if not handles:
            raise ValueError(
                "At least one handle must be provided to commit to the feed"
            )
        elif len(handles) > LIST_FEED_ORDERS_MAX_PAGE_SIZE:
            raise ValueError(
                f"At most {LIST_FEED_ORDERS_MAX_PAGE_SIZE} feed orders can be commited"
                f"at once"
            )

        return self._request(
            method="POST",
            environment=self.ENVIRONMENT,
            endpoint="/api/orders/feed/",
            json={"handles": handles},
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )



from typing import Any

from .._dto import VTEXResponse
from .base import BaseAPI


class LicenseManagerAPI(BaseAPI):
    """
    Client for the License Manager API.
    https://developers.vtex.com/docs/api-reference/license-manager-api
    """

    ENVIRONMENT = "vtexcommercestable"

    def get_account(self, **kwargs: Any) -> VTEXResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/vlm/account",
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )



from http.cookiejar import CookieJar
from typing import (
    IO,
    Any,
    AsyncIterable,
    Iterable,
    Literal,
    Mapping,
    Sequence,
    Tuple,
    Union,
)

from httpx import Cookies, Headers, QueryParams


class UndefinedType:
    pass


HTTPMethodType = Literal[
    "DELETE", "GET", "HEAD", "OPTIONS", "PATCH", "POST", "PUT",
    "delete", "get", "head", "options", "patch", "post", "put",
]

PrimitiveTypes = Union[None, bool, int, float, str]
PrimitiveSequenceType = Sequence[PrimitiveTypes]

JSONType = Union[PrimitiveTypes, Sequence["JSONType"], Mapping[str, "JSONType"]]

RequestContent = Union[str, bytes, Iterable[bytes], AsyncIterable[bytes]]

RequestData = Mapping[str, Any]

FileContent = Union[IO[bytes], bytes, str]
FileTypes = Union[
    FileContent,
    Tuple[Union[str, None], FileContent],
    Tuple[Union[str, None], FileContent, Union[str, None]],
    Tuple[Union[str, None], FileContent, Union[str, None], Mapping[str, str]],
]
RequestFiles = Union[Mapping[str, FileTypes], Sequence[Tuple[str, FileTypes]]]

QueryParamTypes = Union[
    QueryParams,
    Mapping[str, Union[PrimitiveTypes, PrimitiveSequenceType]],
    Sequence[Tuple[str, PrimitiveTypes]],
    Tuple[Tuple[str, PrimitiveTypes], ...],
    str,
    bytes,
]

HeaderTypes = Union[
    Headers,
    Mapping[str, str],
    Mapping[bytes, bytes],
    Sequence[Tuple[str, str]],
    Sequence[Tuple[bytes, bytes]],
]

CookieTypes = Union[Cookies, CookieJar, Mapping[str, str], Sequence[Tuple[str, str]]]



from typing import Any

from .._constants import LIST_TRANSACTION_INTERACTIONS_RETRIES
from .._dto import VTEXListResponse
from .base import BaseAPI


class TransactionsAPI(BaseAPI):
    """
    Client for the Transactions API.
    This is not in the VTEX API documentation.
    """

    ENVIRONMENT = "vtexpayments"

    def list_transaction_interactions(
        self,
        transaction_id: str,
        **kwargs: Any,
    ) -> VTEXListResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint=f"/api/pvt/transactions/{transaction_id}/interactions/",
            config=self._config.with_overrides(
                retries=LIST_TRANSACTION_INTERACTIONS_RETRIES,
                **kwargs,
            ),
            response_class=VTEXListResponse,
        )



from . import _constants as VTEXConstants  # noqa: N812
from ._exceptions import VTEXError, VTEXRequestError, VTEXResponseError
from ._vtex import VTEX

__all__ = [
    "VTEX",
    "VTEXConstants",
    "VTEXError",
    "VTEXRequestError",
    "VTEXResponseError",
]


for name in __all__:
    locals()[name].__module__ = "vtex"



from typing import Any, Type, Union

from httpx._types import(
    CookieTypes,
    HeaderTypes,
    QueryParamTypes,
    RequestContent,
    RequestData,
    RequestFiles,
)

from .._dto import VTEXResponseType
from .._types import HTTPMethodType
from .base import BaseAPI


class CustomAPI(BaseAPI):
    """
    Client for calling endpoints that have not yet been implemented by the SDK.
    You can directly call the `request` method to call any VTEX API.
    """

    def request(
        self,
        method: HTTPMethodType,
        environment: str,
        endpoint: str,
        headers: Union[HeaderTypes, None] = None,
        cookies: Union[CookieTypes, None] = None,
        params: Union[QueryParamTypes, None] = None,
        json: Union[Any, None] = None,
        data: Union[RequestData, None] = None,
        content: Union[RequestContent, None] = None,
        files: Union[RequestFiles, None] = None,
        response_class: Union[Type[VTEXResponseType], None] = None,
        **kwargs: Any,
    ) -> VTEXResponseType:
        return self._request(
            method=method,
            environment=environment,
            endpoint=endpoint,
            headers=headers,
            cookies=cookies,
            params=params,
            json=json,
            data=data,
            content=content,
            files=files,
            config=self._config.with_overrides(**kwargs),
            response_class=response_class,
        )



from typing import Any

from .._constants import (
    LIST_CARRIERS_MAX_PAGE_SIZE,
    LIST_CARRIERS_START_PAGE,
    LIST_DOCKS_MAX_PAGE_SIZE,
    LIST_DOCKS_START_PAGE,
    LIST_SHIPPING_POLICIES_MAX_PAGE_SIZE,
    LIST_SHIPPING_POLICIES_START_PAGE,
    MIN_PAGE_SIZE,
)
from .._dto import VTEXPaginatedListResponse, VTEXResponse
from .base import BaseAPI


class LogisticsAPI(BaseAPI):
    """
    Client for the Logistics API.
    https://developers.vtex.com/docs/api-reference/logistics-api
    """

    ENVIRONMENT = "vtexcommercestable"

    def list_shipping_policies(
        self,
        page: int = LIST_SHIPPING_POLICIES_START_PAGE,
        page_size: int = LIST_SHIPPING_POLICIES_MAX_PAGE_SIZE,
        **kwargs: Any,
    ) -> VTEXPaginatedListResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/logistics/pvt/shipping-policies",
            params={
                "page": max(page, LIST_SHIPPING_POLICIES_START_PAGE),
                "perPage": max(
                    min(page_size, LIST_SHIPPING_POLICIES_MAX_PAGE_SIZE),
                    MIN_PAGE_SIZE,
                ),
            },
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXPaginatedListResponse,
        )

    def get_shipping_policy(
        self,
        shipping_policy_id: str,
        **kwargs: Any,
    ) -> VTEXResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint=f"/api/logistics/pvt/shipping-policies/{shipping_policy_id}",
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )

    def list_carriers(
        self,
        page: int = LIST_CARRIERS_START_PAGE,
        page_size: int = LIST_CARRIERS_MAX_PAGE_SIZE,
        **kwargs: Any,
    ) -> VTEXPaginatedListResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/logistics/pvt/configuration/carriers",
            params={
                "page": max(page, LIST_CARRIERS_START_PAGE),
                "perPage": max(
                    min(page_size, LIST_CARRIERS_MAX_PAGE_SIZE),
                    MIN_PAGE_SIZE,
                ),
            },
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXPaginatedListResponse,
        )

    def get_carrier(
        self,
        carrier_id: str,
        **kwargs: Any,
    ) -> VTEXResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint=f"/api/logistics/pvt/configuration/carriers/{carrier_id}",
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )

    def list_docks(
        self,
        page: int = LIST_DOCKS_START_PAGE,
        page_size: int = LIST_DOCKS_MAX_PAGE_SIZE,
        **kwargs: Any,
    ) -> VTEXPaginatedListResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint="/api/logistics/pvt/configuration/docks",
            params={
                "page": max(page, LIST_DOCKS_START_PAGE),
                "perPage": max(
                    min(page_size, LIST_DOCKS_MAX_PAGE_SIZE),
                    MIN_PAGE_SIZE,
                ),
            },
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXPaginatedListResponse,
        )

    def get_dock(self, dock_id: str, **kwargs: Any) -> VTEXResponse:
        return self._request(
            method="GET",
            environment=self.ENVIRONMENT,
            endpoint=f"/api/logistics/pvt/configuration/docks/{dock_id}",
            config=self._config.with_overrides(**kwargs),
            response_class=VTEXResponse,
        )



from typing import Any, Dict, Union

from ._types import JSONType


class VTEXError(Exception):
    def __init__(self, *args: Any, **kwags: Any) -> None:
        super().__init__(*args, **kwags)


class VTEXRequestError(VTEXError):
    def __init__(
        self,
        *args: Any,
        exception: Union[Exception, None] = None,
        method: Union[str, None] = None,
        url: Union[str, None] = None,
        headers: Union[Dict[str, str], None] = None,
        **kwargs: Any,
    ) -> None:
        self.exception = exception
        self.method = method
        self.url = url
        self.headers = headers

        super().__init__(str(exception or "VTEXRequestError"), *args, **kwargs)


class VTEXResponseError(VTEXError):
    def __init__(
        self,
        *args: Any,
        method: Union[str, None] = None,
        url: Union[str, None] = None,
        request_headers: Union[Dict[str, str], None] = None,
        status: Union[int, None] = None,
        data: JSONType = None,
        response_headers: Union[Dict[str, str], None] = None,
        **kwargs: Any,
    ) -> None:
        self.method = method
        self.url = url
        self.request_headers = request_headers
        self.status = status
        self.data = data
        self.response_headers = response_headers

        super().__init__(str(data or "VTEXResponseError"), *args, **kwargs)
